// Copyright 2015, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Author: Zhiwei Zhang <zhiwei.zhang@saturn.uni-freiburg.de>

#ifndef ZHIWEI_ZHANG_SOURCE_OTHER_VARIABLEBYTEENCODING_H_
#define ZHIWEI_ZHANG_SOURCE_OTHER_VARIABLEBYTEENCODING_H_

#include <stdint.h>
#include <stddef.h>
#include <vector>
#include <string>
#include <iostream>
#include <bitset>
#include "../Timer.h"

using std::vector;
using std::bitset;

class VariableByteEncodingVector {
  public:

  typedef vector<char> Bytevector;

  // Encodes the gaps in @list via vbyte encoding.
  template<typename Numeric>
  static void encode(const vector<Numeric>& plaintext,
          Bytevector& encoded, const bool& show = false) {
    Timer t;
    t.start();
    encoded.resize(plaintext.size() * sizeof(Numeric));
    int k = 0;
    for (size_t i = 0; i < plaintext.size(); ++i) {
      /* since I use a gao list as input, so we don't need to calculate the gap.
       * if original list is given as input, just remove the comment mark and 
       * add coment mark at line 37.
       *
       * int x = (i == 0 ? plaintext[0] : plaintext[i] - plaintext[i - 1]);
       *
       */
      int x = plaintext[i];
      do {
        int mod = x % 128;
        x = x / 128;
        encoded[k++] = (x == 0 ? mod : mod + 128);
      } while (x > 0);
    }
    encoded.resize(k);
    t.stop();
    if (show) {
      // sizeof(Numeric) * k
      std::cout << plaintext.size() << "\t" << k << "\t"
      << t.getUSecs() << "\t";
    }
  }

  // Assumes @bits contains the gaps of a Doclist encoded with vbyte,
  // Restores the Doclist.
  template<typename Numeric>
  static void decode(const Bytevector& bytes,
     vector<Numeric>& decoded, const bool& show = false) {
    Timer t;
    t.start();
    decoded.reserve(bytes.size());
    size_t k = 0;
    /* since I use a gao list as input of compress, so we just need
     * to calculate the gap.
     * if original list is given as input of compress, just remove 
     * the comment mark and add coment mark at line 78 - 85.
     *
     * int y = 0;
     * while (k < bytes.size()) {
     *   int x = 0;
     *   int f = 1;
     *  while (bytes[k] < 0) {
     *    x += f * (bytes[k++] + 128);
     *    f *= 128;
     *  }
     *  x += f * bytes[k++];
     *  y += x;
     *  decoded.push_back(y);
     *
     */
    while (k < bytes.size()) {
      int x = 0;
      int f = 1;
      while (bytes[k] < 0) {
        x += f * (bytes[k++] + 128);
        f *= 128;
      }
      x += f * bytes[k++];
      decoded.push_back(x);
    }
    t.stop();
    if (show) std::cout << t.getUSecs() << std::endl;
  }

  // Helper for tests, etc.
  static std::string byteVectorToString(const Bytevector& bytes) {
    std::ostringstream os;
    for (size_t i = 0; i < bytes.size(); ++i) {
      bitset<8> x(bytes[i]);
      os << x;
      if (i < bytes.size() - 1) os << ' ';
    }
    return os.str();
  }
};

#endif  // ZHIWEI_ZHANG_SOURCE_OTHER_VARIABLEBYTEENCODING_H_
