// Copyright 2015, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Author: Zhiwei Zhang

#include <gtest/gtest.h>
#include <string>
#include <vector>
#include "../source/Other/EliasGammaEncodingVector.h"

using std::string;
using std::to_string;
using std::vector;

// _____________________________________________________________________________
TEST(EliasGammaEncodingVectorTest, encode) {
  vector<int> list;
  Bitvector result;
  string resultToString;

  {
    list.push_back(1);
    EliasGammaEncodingVector::encode(list, result);
    resultToString += to_string(result[0]);
    ASSERT_EQ("1", resultToString);
  }

  resultToString.clear();
  result.clear();
  list.clear();

  {
    list.push_back(150);
    // as elias gamma: 000000010010110
    EliasGammaEncodingVector::encode(list, result);
    for (int i = 0; i < result.size(); i++) {
      resultToString += to_string(result[i]);
    }
    ASSERT_EQ("000000010010110", resultToString);
  }

  resultToString.clear();
  result.clear();
  list.clear();

  {
    list.push_back(1);
    list.push_back(150);
    // Gaps 1 and 150 like before
    EliasGammaEncodingVector::encode(list, result);
    for (int i = 0; i < result.size(); i++) {
      resultToString += to_string(result[i]);
    }
    ASSERT_EQ("1000000010010110", resultToString);
  }
}

// _____________________________________________________________________________
TEST(EliasGammaEncodingVectorTest, encodeDecode) {
  vector<int> plaintext;
  vector<int> decoded;
  Bitvector encoded;

  plaintext.push_back(1);
  plaintext.push_back(5);
  plaintext.push_back(16400);
  plaintext.push_back(20000);
  plaintext.push_back(70000);
  decoded.reserve(5);
  
  EliasGammaEncodingVector::encode(plaintext, encoded);
  EliasGammaEncodingVector::decode(encoded, decoded);
  ASSERT_EQ(plaintext.size(), decoded.size());
  for (size_t i = 0; i < plaintext.size(); ++i) {
    ASSERT_EQ(plaintext[i], decoded[i]);
  }
}
