// Copyright 2015, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Author: Zhiwei Zhang

#include <random>
#include <gtest/gtest.h>
#include <vector>
#include "../source/GenerateNumber.h"

using std::vector;

// ________________________________________________________________
TEST(GenerateNumberTest, uniformDistributionPointer) {
  GenerateNumber gn;
  int* result = new int[30];
  gn.uniformDistribution(1, 100, 30, result);
  for (size_t i = 0; i < 30; i++) {
    ASSERT_EQ(true, (result[i] >= 1) && (result[i] <= 100));
  }
  delete[] result;
}

// ________________________________________________________________
TEST(GenerateNumberTest, uniformDistributionVector) {
  vector<int> result;
  GenerateNumber::uniformDistribution(1, 100, 30, result);
  ASSERT_EQ(30, result.size());
  for (int num : result) {
    ASSERT_EQ(true, (num >= 1) && (num <= 100));
  }
}

// ________________________________________________________________
TEST(GenerateNumberTest, geometricDistributionPointer) {
  GenerateNumber gn;
  int* result = new int[30];
  gn.geometricDistribution(0.6, 30, result);
  for (size_t i = 0; i < 30; i++) {
    ASSERT_EQ(true, (result[i] >= 0) && (result[i] <= 5));
  }
  delete[] result;
}

// ________________________________________________________________
TEST(GenerateNumberTest, geometricDistributionVector) {
  vector<unsigned char> result;
  GenerateNumber::geometricDistribution(0.6, 30, result);
  ASSERT_EQ(30, result.size());
  for (unsigned char num : result) {
    ASSERT_EQ(true, (num >= 0) && (num <= 5));
  }
};
