// Copyright 2015, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Author: Zhiwei Zhang

#include <gtest/gtest.h>
#include <vector>
#include "../source/Other/VariableByteEncodingVector.h"

using std::vector;

// _____________________________________________________________________________
TEST(VariableByteEncodingVectorTest, encode) {
  vector<int> list;
  VariableByteEncodingVector::Bytevector result;

  {
    list.push_back(1);
    VariableByteEncodingVector::encode(list, result);
    ASSERT_EQ("00000001",
              VariableByteEncodingVector::byteVectorToString(result));
  }

  result.clear();
  list.clear();

  {
    list.push_back(150);
    VariableByteEncodingVector::encode(list, result);
    // 150 = 22 * 128^0 + 1 * 128^1
    // as vbyte: 10010110 00000001
    ASSERT_EQ("10010110 00000001",
              VariableByteEncodingVector::byteVectorToString(result));
  }

  result.clear();
  list.clear();

  {
    list.push_back(1);
    list.push_back(150);
    VariableByteEncodingVector::encode(list, result);
    // Gaps 1 and 150 like before
    ASSERT_EQ("00000001 10010110 00000001",
        VariableByteEncodingVector::byteVectorToString(result));
  }
}

// _____________________________________________________________________________
TEST(VariableByteEncodingVectorTest, encodeDecode) {
  vector<int> plaintext;
  vector<int> decoded;
  VariableByteEncodingVector::Bytevector encoded;

  plaintext.push_back(1);
  plaintext.push_back(5);
  plaintext.push_back(16400);
  plaintext.push_back(20000);
  plaintext.push_back(70000);

  VariableByteEncodingVector::encode(plaintext, encoded);
  VariableByteEncodingVector::decode(encoded, decoded);
  ASSERT_EQ(plaintext.size(), decoded.size());
  for (size_t i = 0; i < plaintext.size(); ++i) {
    ASSERT_EQ(plaintext[i], decoded[i]);
  }
}
